export_logs

Function export_logs 

pub fn export_logs(dest: &Path) -> Result<()>
Bundle the current session log and the most recent crash report (plus a version/OS header) into a single file at dest, for the user to attach to a bug report. Triggered by the settings “Export logs” button.