MarkdownFile

Struct MarkdownFile 

pub struct MarkdownFile<T: DeserializeOwned> {
    pub frontmatter: Option<T>,
    pub content: String,
}
Represents a parsed markdown file with optional frontmatter

Fields

frontmatter: Option<T>

Parsed frontmatter (if present)

content: String

The content after frontmatter

Implementations

Source

impl<T: DeserializeOwned + Send + 'static> MarkdownFile<T>

pub fn parse(path: impl AsRef<Path>) -> Result<Self, ParseError>

pub fn list(dir: impl AsRef<Path>) -> Result<Vec<PathBuf>, Error>

List all markdown files in a directory

pub fn from_file(path: impl AsRef<Path>) -> Result<Self, ParseError>

Load a single markdown file from a path

pub async fn from_dir(dir: &PathBuf) -> Result<Vec<(PathBuf, Self)>, Error>

Load all markdown files from a directory

pub async fn from_nested_dirs( parent_dir: impl AsRef<Path>, filename: &str, ) -> Result<Vec<(PathBuf, Self)>, Error>

Load all markdown files from nested subdirectories, where each subdirectory contains a file with the specified filename.

Flat files in the parent directory are ignored. Only subdirectories containing the specified filename are processed.

Example
// Load from:
//   skills/skill-1/SKILL.md
//   skills/skill-2/SKILL.md
//   skills/flat-file.md      -> ignored (not in a subdirectory)
let skills = MarkdownFile::from_nested_dirs(Path::new("skills"), "SKILL.md").await?;