is_false

Function is_false 

pub fn is_false(b: &bool) -> bool
Serde skip_serializing_if helper for bool fields that default to false.

Takes &bool rather than bool because skip_serializing_if calls the predicate with a reference to the field.