TinyStr

Struct TinyStr 

pub struct TinyStr<T: TinyStrInt>(/* private fields */);
A short string packed into a big-endian NonZero integer.

TinyStr stores up to N bytes in a single value that fits in a register, enabling single-instruction equality and ordering comparisons. The big-endian layout means normal integer comparisons result in lexicographic order.

Strings are NUL-padded. Trailing NUL bytes are indistinguishable from padding, so two inputs only differing in trailing NULs will be equal.

Empty strings or strings containing all NUL bytes are not representable.

Examples

let s4 = TinyStr4::new(b"abc").unwrap();
assert_eq!(s4.len(), 3);
assert_eq!(format!("{s4}"), "abc");
assert!(s4 < TinyStr4::from_const(b"abd"));
assert!(s4 > TinyStr4::from_const(b"abb"));

let s8 = TinyStr8::new(b"abcdefg").unwrap();
assert_eq!(s8.len(), 7);
assert_eq!(format!("{s8}"), "abcdefg");
assert_eq!(s8, const { TinyStr8::from_const(b"abcdefg") });

Implementations

Source

impl<T: TinyStrInt> TinyStr<T>

pub fn new(s: &[u8]) -> Option<Self>

Creates a new TinyStr from a byte slice.

Returns None if the slice is empty, only contains NUL bytes, or is too long.

Examples
let s = TinyStr4::new(b"abc").unwrap();
assert_eq!(s.len(), 3);
assert_eq!(format!("{s}"), "abc");

assert!(TinyStr4::new(b"").is_none());
assert!(TinyStr4::new(b"abcde").is_none());
pub const fn from_raw(raw: T) -> Self

Creates a new TinyStr from a raw NonZero integer.

Examples
let raw = NonZero::new(0x61626300).unwrap();
let s = TinyStr4::from_raw(raw);
assert_eq!(s.len(), 3);
assert_eq!(format!("{s}"), "abc");
pub fn len(self) -> usize

Returns the number of bytes in the string.

Examples
assert_eq!(TinyStr4::new(b"a").unwrap().len(), 1);
assert_eq!(TinyStr4::new(b"ab").unwrap().len(), 2);
assert_eq!(TinyStr4::new(b"abc").unwrap().len(), 3);
assert_eq!(TinyStr4::new(b"abcd").unwrap().len(), 4);
Source

impl TinyStr<NonZero<u16>>

pub const fn from_const(s: &[u8]) -> Self

Creates a TinyStr from a byte slice at compile time, panicking if the string is invalid.

Examples
const S: TinyStr<NonZero<u16>> = TinyStr::<NonZero<u16>>::from_const(b"ab");
assert_eq!(S, TinyStr::new(b"ab").unwrap());
assert_eq!(S.to_string(), "ab");
assert_eq!(S.len(), 2);
Source

impl TinyStr<NonZero<u32>>

pub const fn from_const(s: &[u8]) -> Self

Creates a TinyStr from a byte slice at compile time, panicking if the string is invalid.

Examples
const S: TinyStr<NonZero<u32>> = TinyStr::<NonZero<u32>>::from_const(b"ab");
assert_eq!(S, TinyStr::new(b"ab").unwrap());
assert_eq!(S.to_string(), "ab");
assert_eq!(S.len(), 2);
Source

impl TinyStr<NonZero<u64>>

pub const fn from_const(s: &[u8]) -> Self

Creates a TinyStr from a byte slice at compile time, panicking if the string is invalid.

Examples
const S: TinyStr<NonZero<u64>> = TinyStr::<NonZero<u64>>::from_const(b"ab");
assert_eq!(S, TinyStr::new(b"ab").unwrap());
assert_eq!(S.to_string(), "ab");
assert_eq!(S.len(), 2);