pub trait TinyStrLen<const N: usize> {
type Int: TinyStrInt;
}Expand description
Helper trait to map between length N and the smallest TinyStrInt type that can store it.
See parser::tinystr.
pub trait TinyStrLen<const N: usize> {
type Int: TinyStrInt;
}Helper trait to map between length N and the smallest TinyStrInt type that can store it.
See parser::tinystr.