pub fn is_false(b: &bool) -> bool
Serde skip_serializing_if helper for bool fields that default to false.
skip_serializing_if
bool
false
Takes &bool rather than bool because skip_serializing_if calls the predicate with a reference to the field.
&bool